﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.IO;

namespace SimpleTemplate
{
  public partial class MainForm : Form
  {
    private List<String> sourceFileNames = new List<string>();
    private Dictionary<String, String> templateValues = new Dictionary<string, string>();

    public MainForm()
    {
      InitializeComponent();

      ofSelectSourceFiles.InitialDirectory = System.Environment.CurrentDirectory;
    }

    private void btnSelectSourceFiles_Click(object sender, EventArgs e)
    {
      if (ofSelectSourceFiles.ShowDialog() == DialogResult.OK)
      {
        sourceFileNames = new List<string>();

        foreach (String file in ofSelectSourceFiles.FileNames)
          sourceFileNames.Add(file);

        determineTemplateValues();
        createTemplateControls();
        refreshSourceFiles();
      }
    }

    private void refreshSourceFiles()
    {
      lbSelectedSourceFiles.Items.Clear();
      foreach (String s in sourceFileNames)
      {
        lbSelectedSourceFiles.Items.Add(s);
      }
      lbSelectedSourceFiles.Refresh();
      Application.DoEvents();
    }

    private void createTemplateControls()
    {
      panelValues.Controls.Clear();

      int i = 0;
      foreach (String key in templateValues.Keys)
      {
        Label lbl = new Label();
        lbl.Name = String.Format("lblValue{0}", i);
        lbl.Text = key;
        lbl.Location = new Point(12, i * 24);

        TextBox tb = new TextBox();
        tb.Name = String.Format("tbValue{0}", i);
        tb.Tag = key;
        tb.Text = templateValues[key];
        tb.Location = new Point(256, i * 24);
        tb.Size = new Size(256, 20);
        tb.GotFocus += new EventHandler(tb_GotFocus);
        tb.TextChanged += new EventHandler(tb_TextChanged);

        panelValues.Controls.Add(lbl);
        panelValues.Controls.Add(tb);

        i++;
      }

      panelValues.Refresh();
      Application.DoEvents();
    }

    void tb_GotFocus(object sender, EventArgs e)
    {
      (sender as TextBox).SelectAll();
    }

    void tb_TextChanged(object sender, EventArgs e)
    {
      String key = (String)(sender as TextBox).Tag;
      templateValues[key] = (sender as TextBox).Text;
    }

    private void addMatches(MatchCollection matches)
    {
      foreach (Match m in matches)
      {
        if (!templateValues.ContainsKey(m.Value))
        {
          templateValues.Add(m.Value, m.Value);
        }
      }
    }

    private void determineTemplateValues()
    {
      Regex rexTemp = new Regex("(###)([^#]*)(###)");

      foreach (String sfName in sourceFileNames)
      {
        MatchCollection mcFname = rexTemp.Matches(sfName);
        addMatches(mcFname);

        StreamReader sr = new StreamReader(sfName);
        String source = sr.ReadToEnd();

        MatchCollection mcSource = rexTemp.Matches(source);
        addMatches(mcSource);
      }
    }

    private String getReplacedString(String source)
    {
      String copy = String.Copy(source);
      foreach (String key in templateValues.Keys)
      {
        copy = copy.Replace(key, templateValues[key]);
      }
      return copy;
    }

    private void btnGo_Click(object sender, EventArgs e)
    {
      foreach (String sf in sourceFileNames)
      {
        String tf = getReplacedString(sf);

        if (  (File.Exists(tf) &&
               MessageBox.Show(String.Format("Overwrite existing file '{0}'?", tf), 
                               "Warning!", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
               || !File.Exists(tf))
          {
            StreamReader sr = new StreamReader(sf);
            StreamWriter sw = new StreamWriter(tf, false);

            while (!sr.EndOfStream)
            {
              String sourceline = sr.ReadLine();
              String targetline = getReplacedString(sourceline);
              sw.WriteLine(targetline);
            }

            sw.Flush();
            sw.Close();

            sr.Close();
          }
      }
    }
  }
}
